@@ -553,54 +553,45 @@ CONTENT\r
 #[test]
 fn test_get_quoted_utf16() {
     fn es2020(src: &str, expected: &str) {
-        assert_eq!(
-            super::get_quoted_utf16(src, EsVersion::Es2020, true, false, false),
-            expected
-        )
+        assert_eq!(super::get_quoted_utf16(src, EsVersion::Es2020), expected)
     }
 
     fn es2020_nonascii(src: &str, expected: &str) {
-        assert_eq!(
-            super::get_quoted_utf16(src, EsVersion::Es2020, true, true, false),
-            expected
-        )
+        assert_eq!(super::get_quoted_utf16(src, EsVersion::Es2020), expected)
     }
 
     fn es5(src: &str, expected: &str) {
-        assert_eq!(
-            super::get_quoted_utf16(src, EsVersion::Es5, true, true, false),
-            expected
-        )
+        assert_eq!(super::get_quoted_utf16(src, EsVersion::Es5), expected)
     }
 
-    es2020("abcde", "abcde");
+    es2020("abcde", "\"abcde\"");
     es2020(
         "\x00\r\n\u{85}\u{2028}\u{2029};",
-        "\\0\\r\\n\\x85\\u2028\\u2029;",
+        "\"\\0\\r\\n\\x85\\u2028\\u2029;\"",
     );
 
-    es2020("\n", "\\n");
-    es2020("\t", "\\t");
+    es2020("\n", "\"\\n\"");
+    es2020("\t", "\"\\t\"");
 
-    es2020("'string'", "\\'string\\'");
+    es2020("'string'", "\"'string'\"");
 
-    es2020("\u{0}", "\\0");
-    es2020("\u{1}", "\\x01");
+    es2020("\u{0}", "\"\\0\"");
+    es2020("\u{1}", "\"\\x01\"");
 
-    es2020("\u{1000}", "\\u1000");
-    es2020("\u{ff}", "\\xff");
-    es2020("\u{10ffff}", "\\u{10FFFF}");
-    es2020("😀", "\\u{1F600}");
-    es5("ퟻ", "\\uD7FB");
+    es2020("\u{1000}", "\"\\u1000\"");
+    es2020("\u{ff}", "\"\\xff\"");
+    es2020("\u{10ffff}", "\"\\u{10FFFF}\"");
+    es2020("😀", "\"\\u{1F600}\"");
+    es5("ퟻ", "\"\\uD7FB\"");
 
-    es2020_nonascii("\u{FEFF}abc", "\\uFEFFabc");
-    es2020_nonascii("\u{10ffff}", "\\u{10FFFF}");
+    es2020_nonascii("\u{FEFF}abc", "\"\\uFEFFabc\"");
+    es2020_nonascii("\u{10ffff}", "\"\\u{10FFFF}\"");
 
-    es5("\u{FEFF}abc", "\\uFEFFabc");
-    es5("\u{10ffff}", "\\uDBFF\\uDFFF");
-    es5("\u{FFFF}", "\\uFFFF");
-    es5("😀", "\\uD83D\\uDE00");
-    es5("ퟻ", "\\uD7FB");
+    es5("\u{FEFF}abc", "\"\\uFEFFabc\"");
+    es5("\u{10ffff}", "\"\\uDBFF\\uDFFF\"");
+    es5("\u{FFFF}", "\"\\uFFFF\"");
+    es5("😀", "\"\\uD83D\\uDE00\"");
+    es5("ퟻ", "\"\\uD7FB\"");
 }
 
 #[test]
@@ -636,10 +627,7 @@ fn issue_1619_2() {
 
 #[test]
 fn issue_1619_3() {
-    assert_eq!(
-        get_quoted_utf16("\x00\x31", EsVersion::Es3, true, false, false),
-        "\\x001"
-    );
+    assert_eq!(get_quoted_utf16("\x00\x31", EsVersion::Es3), "\"\\x001\"");
 }
 
 fn check_latest(src: &str, expected: &str) {
@@ -685,11 +673,12 @@ fn issue_2213() {
 
 #[test]
 fn issue3617() {
+    // Convert characters to es5 compatibility code
     let from = r"// a string of all valid unicode whitespaces
     module.exports = '\u0009\u000A\u000B\u000C\u000D\u0020\u00A0\u1680\u2000\u2001\u2002' +
-      '\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF';";
+      '\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF' + '\u{a0}';";
     let expected = r#"// a string of all valid unicode whitespaces
-module.exports = '\t\n\v\f\r \xa0\u1680\u2000\u2001\u2002' + '\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF';"#;
+module.exports = "\t\n\v\f\r \xa0\u1680\u2000\u2001\u2002" + "\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF" + "\xa0";"#;
 
     let out = parse_then_emit(from, Default::default(), Syntax::default(), EsVersion::Es5);
 
@@ -701,3 +690,28 @@ module.exports = '\t\n\v\f\r \xa0\u1680\u2000\u2001\u2002' + '\u2003\u2004\u2005
         DebugUsingDisplay(expected.trim()),
     );
 }
+
+#[test]
+fn issue3617_1() {
+    // Print characters as is for ECMA target > 5
+    let from = r"// a string of all valid unicode whitespaces
+    module.exports = '\u0009\u000A\u000B\u000C\u000D\u0020\u00A0\u1680\u2000\u2001\u2002' +
+      '\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF' + '\u{a0}';";
+    let expected = r#"// a string of all valid unicode whitespaces
+module.exports = '\u0009\u000A\u000B\u000C\u000D\u0020\u00A0\u1680\u2000\u2001\u2002' + '\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF' + '\u{a0}';"#;
+
+    let out = parse_then_emit(
+        from,
+        Default::default(),
+        Syntax::default(),
+        EsVersion::Es2022,
+    );
+
+    dbg!(&out);
+    dbg!(&expected);
+
+    assert_eq!(
+        DebugUsingDisplay(out.trim()),
+        DebugUsingDisplay(expected.trim()),
+    );
+}
